@@ -94,7 +94,7 @@ int cch__find(struct cache_t *cache, cache_value_t id,
 		int *index, char **data, int *len);
 
 /** Copy the given data into the given cache entry. */
-inline int cch__entry_set(struct cache_entry_t **cache, 
+int cch__entry_set(struct cache_entry_t **cache, 
 		cache_value_t id, const char *data, int len,
 		int copy_old_data,
 		char **copy);
